@@ -1,14 +1,16 @@
+import unittest
+
 from twisted.internet import reactor
 from twisted.internet.error import ConnectError, ConnectionDone
 from twisted.internet.task import Clock
 from twisted.internet.defer import Deferred, inlineCallbacks
 from twisted.python.failure import Failure
 
+from landscape.lib import testing
 from landscape.lib.amp import (
     MethodCallError, MethodCallServerProtocol, MethodCallClientProtocol,
     MethodCallServerFactory, MethodCallClientFactory, RemoteObject,
     MethodCallSender)
-from landscape.tests.helpers import LandscapeTest
 
 
 class FakeTransport(object):
@@ -22,7 +24,7 @@ class FakeTransport(object):
         self.stream.append(data)
 
     def loseConnection(self):
-        raise NotImplemented()
+        raise NotImplementedError()
 
     def getPeer(self):
         pass
@@ -92,7 +94,11 @@ class DummyObject(object):
     method = None
 
 
-class MethodCallTest(LandscapeTest):
+class BaseTestCase(testing.TwistedTestCase, unittest.TestCase):
+    pass
+
+
+class MethodCallTest(BaseTestCase):
 
     def setUp(self):
         super(MethodCallTest, self).setUp()
@@ -204,12 +210,29 @@ class MethodCallTest(LandscapeTest):
         """
         Method arguments passed to a L{MethodCall} can be dictionaries.
         """
-        self.object.method = lambda d: "".join(d.keys()) * sum(d.values())
+        # Sort the keys to ensure stable test outcome.
+        self.object.method = lambda d: "".join(
+            sorted(d.keys()) * sum(d.values()))
         deferred = self.sender.send_method_call(method="method",
                                                 args=[{"foo": 1, "bar": 2}],
                                                 kwargs={})
         self.connection.flush()
-        self.assertEqual("foobarfoobarfoobar", self.successResultOf(deferred))
+        self.assertEqual("barfoobarfoobarfoo", self.successResultOf(deferred))
+
+    def test_with_bytes_dictionary_arguments(self):
+        """
+        Method arguments passed to a MethodCall can be a dict of bytes.
+        """
+        arg = {b"byte_key": 1}
+        self.object.method = lambda d: ",".join([
+            type(x).__name__ for x in d.keys()])
+        deferred = self.sender.send_method_call(
+            method="method",
+            args=[arg],
+            kwargs={})
+        self.connection.flush()
+        # str under python2, bytes under python3
+        self.assertEqual(type(b"").__name__, self.successResultOf(deferred))
 
     def test_with_non_serializable_return_value(self):
         """
@@ -355,7 +378,7 @@ class MethodCallTest(LandscapeTest):
         failure.trap(MethodCallError)
 
 
-class RemoteObjectTest(LandscapeTest):
+class RemoteObjectTest(BaseTestCase):
 
     def setUp(self):
         super(RemoteObjectTest, self).setUp()
@@ -462,7 +485,7 @@ class RemoteObjectTest(LandscapeTest):
         self.assertEqual("Forbidden method 'method'", str(failure.value))
 
 
-class MethodCallClientFactoryTest(LandscapeTest):
+class MethodCallClientFactoryTest(BaseTestCase):
 
     def setUp(self):
         super(MethodCallClientFactoryTest, self).setUp()
@@ -556,7 +579,7 @@ class MethodCallClientFactoryTest(LandscapeTest):
         self.assertIs(None, self.successResultOf(deferred))
 
 
-class MethodCallFunctionalTest(LandscapeTest):
+class MethodCallFunctionalTest(BaseTestCase):
 
     def setUp(self):
         super(MethodCallFunctionalTest, self).setUp()
