@@ -1,17 +1,21 @@
 # -*- coding: utf-8 -*-
+import codecs
+import unittest
 
-from landscape.tests.helpers import LandscapeTest
-from landscape.lib.encoding import encode_if_needed, encode_dict_if_needed
+from landscape.lib.encoding import encode_if_needed, encode_values
 
 
-class EncodingTest(LandscapeTest):
+EXPECTED_UTF8 = codecs.encode(u"請不要刪除", "utf-8")
+
+
+class EncodingTest(unittest.TestCase):
 
     def test_encode_if_needed_utf_string(self):
         """
         When passed an utf-8 str() instance the encode_if_needed function
         returns the same.
         """
-        value = "請不要刪除"
+        value = EXPECTED_UTF8
         result = encode_if_needed(value)
         self.assertEqual(value, result)
 
@@ -23,7 +27,7 @@ class EncodingTest(LandscapeTest):
         """
         value = u"Alex \U0001f603"
         result = encode_if_needed(value)
-        expected = 'Alex \xf0\x9f\x98\x83'
+        expected = b'Alex \xf0\x9f\x98\x83'
         self.assertEqual(expected, result)
 
     def test_encode_if_needed_utf_unicode(self):
@@ -32,9 +36,8 @@ class EncodingTest(LandscapeTest):
         the encode_if_needed function returns the utf-8 str() equivalent.
         """
         value = u'\u8acb\u4e0d\u8981\u522a\u9664'
-        expected = "請不要刪除"
         result = encode_if_needed(value)
-        self.assertEqual(expected, result)
+        self.assertEqual(EXPECTED_UTF8, result)
 
     def test_encode_if_needed_utf_unicode_string(self):
         """
@@ -42,9 +45,8 @@ class EncodingTest(LandscapeTest):
         function returns the utf-8 str() equivalent.
         """
         value = u"請不要刪除"
-        expected = "請不要刪除"
         result = encode_if_needed(value)
-        self.assertEqual(expected, result)
+        self.assertEqual(EXPECTED_UTF8, result)
 
     def test_encode_if_needed_with_null_value(self):
         """
@@ -52,14 +54,11 @@ class EncodingTest(LandscapeTest):
         """
         self.assertIs(None, encode_if_needed(None))
 
-    def test_encode_dict_if_needed(self):
+    def test_encode_values(self):
         """
-        The encode_dict_if_needed function returns a dict for which every
-        value was passed to the encode_if_needed function.
+        When passed in a dictionary, all unicode is encoded and bytes are left.
         """
-        value = {"a": "請不要刪除", "b": u'\u8acb\u4e0d\u8981\u522a\u9664',
-                 "c": u"請不要刪除", "d": None, "e": 123}
-        expected = {"a": "請不要刪除", "b": "請不要刪除", "c": "請不要刪除",
-                    "d": None, "e": 123}
-        result = encode_dict_if_needed(value)
-        self.assertEqual(expected, result)
+        original = {"a": b"Alex \xf0\x9f\x98\x83", "b": u"Alex \U0001f603"}
+        expected = {"a": b"Alex \xf0\x9f\x98\x83",
+                    "b": b"Alex \xf0\x9f\x98\x83"}
+        self.assertEqual(expected, encode_values(original))
