@@ -1,13 +1,15 @@
+import mock
 import os
+import unittest
 
 from twisted.internet import reactor
 from twisted.internet.defer import Deferred
 
-from landscape.tests.helpers import LandscapeTest
+from landscape.lib import testing
 from landscape.lib.gpg import gpg_verify
 
 
-class GPGTest(LandscapeTest):
+class GPGTest(testing.FSTestCase, testing.TwistedTestCase, unittest.TestCase):
 
     def test_gpg_verify(self):
         """
@@ -18,17 +20,13 @@ class GPGTest(LandscapeTest):
         gpg = self.makeFile("#!/bin/sh\n"
                             "touch $3/trustdb.gpg\n"
                             "echo -n $@ > %s\n" % gpg_options)
-        os.chmod(gpg, 0755)
+        os.chmod(gpg, 0o755)
         gpg_home = self.makeDir()
-        mkdtemp_mock = self.mocker.replace("tempfile.mkdtemp")
-        mkdtemp_mock()
-        self.mocker.result(gpg_home)
-        self.mocker.replay()
-
         deferred = Deferred()
 
-        def do_test():
-
+        @mock.patch("tempfile.mkdtemp")
+        def do_test(mkdtemp_mock):
+            mkdtemp_mock.return_value = gpg_home
             result = gpg_verify("/some/file", "/some/signature", gpg=gpg)
 
             def check_result(ignored):
@@ -51,17 +49,13 @@ class GPGTest(LandscapeTest):
         provided signature is not valid.
         """
         gpg = self.makeFile("#!/bin/sh\necho out; echo err >&2; exit 1\n")
-        os.chmod(gpg, 0755)
+        os.chmod(gpg, 0o755)
         gpg_home = self.makeDir()
-        mkdtemp_mock = self.mocker.replace("tempfile.mkdtemp")
-        mkdtemp_mock()
-        self.mocker.result(gpg_home)
-        self.mocker.replay()
-
         deferred = Deferred()
 
-        def do_test():
-
+        @mock.patch("tempfile.mkdtemp")
+        def do_test(mkdtemp_mock):
+            mkdtemp_mock.return_value = gpg_home
             result = gpg_verify("/some/file", "/some/signature", gpg=gpg)
 
             def check_failure(failure):
