@@ -1,14 +1,21 @@
-import array
 import socket
 import unittest
 
 from mock import patch, ANY, mock_open
+from netifaces import (
+    AF_INET,
+    AF_INET6,
+    AF_LINK,
+    AF_UNIX,
+    ifaddresses as _ifaddresses,
+    interfaces as _interfaces,
+)
 from subprocess import Popen, PIPE
 
 from landscape.lib import testing
 from landscape.lib.network import (
     get_network_traffic, get_active_device_info, get_active_interfaces,
-    get_fqdn, get_network_interface_speed)
+    get_fqdn, get_network_interface_speed, is_up)
 
 
 class BaseTestCase(testing.HelperTestCase, unittest.TestCase):
@@ -26,9 +33,8 @@ class NetworkInfoTest(BaseTestCase):
         """
         mock_get_network_interface_speed.return_value = (100, True)
 
-        device_info = get_active_device_info()
-        process = Popen(
-            ["/sbin/ifconfig"], stdout=PIPE, env={"LC_ALL": "C"})
+        device_info = get_active_device_info(extended=False)
+        process = Popen(["/sbin/ifconfig"], stdout=PIPE, env={"LC_ALL": "C"})
         result = process.communicate()[0].decode("ascii")
         interface_blocks = dict(
             [(block.split()[0].strip(":"), block.upper()) for block in
@@ -37,9 +43,9 @@ class NetworkInfoTest(BaseTestCase):
         for device in device_info:
             if device["mac_address"] == "00:00:00:00:00:00":
                 continue
-            self.assertTrue(device["interface"] in result)
+            self.assertIn(device["interface"], result)
             block = interface_blocks[device["interface"]]
-            self.assertTrue(device["netmask"] in block)
+            self.assertIn(device["netmask"], block)
             self.assertIn(device["ip_address"], block)
             self.assertIn(device["mac_address"].upper(), block)
             self.assertIn(device["broadcast_address"], block)
@@ -58,6 +64,90 @@ class NetworkInfoTest(BaseTestCase):
         self.assertTrue(mock_get_network_interface_speed.call_count >= 1)
         mock_get_network_interface_speed.assert_called_with(ANY, ANY)
 
+    @patch("landscape.lib.network.get_network_interface_speed")
+    @patch("landscape.lib.network.get_flags")
+    @patch("landscape.lib.network.netifaces.ifaddresses")
+    @patch("landscape.lib.network.netifaces.interfaces")
+    def test_extended_info(self, mock_interfaces, mock_ifaddresses,
+                           mock_get_flags, mock_get_network_interface_speed):
+        mock_get_network_interface_speed.return_value = (100, True)
+        mock_get_flags.return_value = 4163
+        mock_interfaces.return_value = ["test_iface"]
+        mock_ifaddresses.return_value = {
+            AF_LINK: [
+                {"addr": "aa:bb:cc:dd:ee:f0"},
+                {"addr": "aa:bb:cc:dd:ee:f1"}],
+            AF_INET: [
+                {"addr": "192.168.0.50", "netmask": "255.255.255.0"},
+                {"addr": "192.168.1.50", "netmask": "255.255.255.0"}],
+            AF_INET6: [
+                {"addr": "2001::1"},
+                {"addr": "2002::2"}]}
+
+        device_info = get_active_device_info(extended=True)
+
+        self.assertEqual(
+            device_info,
+            [{
+                'interface': 'test_iface',
+                'ip_address': '192.168.0.50',
+                'mac_address': 'aa:bb:cc:dd:ee:f0',
+                'broadcast_address': '0.0.0.0',
+                'netmask': '255.255.255.0',
+                'ip_addresses': {
+                    AF_INET: [
+                        {'addr': '192.168.0.50', 'netmask': '255.255.255.0'},
+                        {'addr': '192.168.1.50', 'netmask': '255.255.255.0'}],
+                    AF_INET6: [
+                        {"addr": "2001::1"},
+                        {"addr": "2002::2"}]},
+                'flags': 4163,
+                'speed': 100,
+                'duplex': True}])
+
+    @patch("landscape.lib.network.get_network_interface_speed")
+    @patch("landscape.lib.network.get_flags")
+    @patch("landscape.lib.network.netifaces.ifaddresses")
+    @patch("landscape.lib.network.netifaces.interfaces")
+    def test_skip_ipv6_only_in_non_extended_mode(
+            self, mock_interfaces, mock_ifaddresses, mock_get_flags,
+            mock_get_network_interface_speed):
+        mock_get_network_interface_speed.return_value = (100, True)
+        mock_get_flags.return_value = 4163
+        mock_interfaces.return_value = ["test_iface"]
+        mock_ifaddresses.return_value = {
+            AF_LINK: [{"addr": "aa:bb:cc:dd:ee:f0"}],
+            AF_INET6: [{"addr": "2001::1"}]}
+
+        device_info = get_active_device_info(extended=False)
+
+        self.assertEqual(device_info, [])
+
+    @patch("landscape.lib.network.get_network_interface_speed")
+    @patch("landscape.lib.network.get_flags")
+    @patch("landscape.lib.network.netifaces.ifaddresses")
+    @patch("landscape.lib.network.netifaces.interfaces")
+    def test_ipv6_only_in_extended_mode(
+            self, mock_interfaces, mock_ifaddresses, mock_get_flags,
+            mock_get_network_interface_speed):
+        mock_get_network_interface_speed.return_value = (100, True)
+        mock_get_flags.return_value = 4163
+        mock_interfaces.return_value = ["test_iface"]
+        mock_ifaddresses.return_value = {
+            AF_LINK: [{"addr": "aa:bb:cc:dd:ee:f0"}],
+            AF_INET6: [{"addr": "2001::1"}]}
+
+        device_info = get_active_device_info(extended=True)
+
+        self.assertEqual(
+            device_info,
+            [{
+                'interface': 'test_iface',
+                'flags': 4163,
+                'speed': 100,
+                'duplex': True,
+                'ip_addresses': {AF_INET6: [{'addr': '2001::1'}]}}])
+
     def test_skip_loopback(self):
         """The C{lo} interface is not reported by L{get_active_device_info}."""
         device_info = get_active_device_info()
@@ -67,76 +157,82 @@ class NetworkInfoTest(BaseTestCase):
     @patch("landscape.lib.network.get_active_interfaces")
     def test_skip_vlan(self, mock_get_active_interfaces):
         """VLAN interfaces are not reported by L{get_active_device_info}."""
-        mock_get_active_interfaces.side_effect = lambda sock: (
-            list(get_active_interfaces(sock)) + [b"eth0.1"])
+        mock_get_active_interfaces.side_effect = lambda: (
+            list(get_active_interfaces()) + [("eth0.1", {})])
         device_info = get_active_device_info()
-        mock_get_active_interfaces.assert_called_with(ANY)
+        self.assertTrue(mock_get_active_interfaces.called)
         interfaces = [i["interface"] for i in device_info]
         self.assertNotIn("eth0.1", interfaces)
 
     @patch("landscape.lib.network.get_active_interfaces")
     def test_skip_alias(self, mock_get_active_interfaces):
         """Interface aliases are not reported by L{get_active_device_info}."""
-        mock_get_active_interfaces.side_effect = lambda sock: (
-            list(get_active_interfaces(sock)) + [b"eth0:foo"])
+        mock_get_active_interfaces.side_effect = lambda: (
+            list(get_active_interfaces()) + [("eth0:foo", {})])
         device_info = get_active_device_info()
         interfaces = [i["interface"] for i in device_info]
         self.assertNotIn("eth0:foo", interfaces)
 
-    @patch("struct.unpack")
-    @patch("fcntl.ioctl")
-    def test_duplicate_network_interfaces(self, mock_ioctl, mock_unpack):
-        """
-        L{get_active_interfaces} doesn't return duplicate network interfaces.
-        The call to C{fcntl.ioctl} might return the same interface several
-        times, so we make sure to clean it up.
-        """
-        import landscape.lib.network
-        original_struct_size = landscape.lib.network.IF_STRUCT_SIZE
-        landscape.lib.network.IF_STRUCT_SIZE = 40
-        self.addCleanup(
-            setattr, landscape.lib.network, "IF_STRUCT_SIZE",
-            original_struct_size)
-        # This is a fake response observed to return the same interface several
-        # times (here, br1:priv)
-        response = (
-            b"lo\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02"
-            b"\x00\x00\x00\x7f\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00"
-            b"\x00\x00\x00\x00\x00\x00\x00eth1:pub\x00\x00\x00\x00\x00\x00\x00"
-            b"\x00\x02\x00\x00\x00\xc8\xb4\xc4.\x00\x00\x00\x00\x00\x00\x00"
-            b"\x00"
-            b"\x00\x00\x00\x00\x00\x00\x00\x00br1:metadata\x00\x00\x00\x00\x02"
-            b"\x00\x00\x00\xa9\xfe\xa9\xfe\x00\x00\x00\x00\x00\x00\x00\x00\x00"
-            b"\x00\x00\x00\x00\x00\x00\x00br1:0\x00\x00\x00\x00\x00\x00\x00"
-            b"\x00"
-            b"\x00\x00\x00\x02\x00\x00\x00\xc9\x19\x1f\x1d\x00\x00\x00\x00\x00"
-            b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00br1\x00\x00\x00\x00"
-            b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00\x00\xc0\xa8d"
-            b"\x1d\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
-            b"\x00br1:priv\x00\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00\x00\xac"
-            b"\x13\x02\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
-            b"\x00\x00\x00br1:priv\x00\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00"
-            b"\x00\xac\x13\x02A")
+    @patch("landscape.lib.network.netifaces.ifaddresses")
+    @patch("landscape.lib.network.netifaces.interfaces")
+    def test_skip_iface_with_no_addr(self, mock_interfaces, mock_ifaddresses):
+        mock_interfaces.return_value = _interfaces() + ["test_iface"]
+        mock_ifaddresses.side_effect = lambda iface: (
+            _ifaddresses(iface) if iface in _interfaces() else {})
+        device_info = get_active_device_info()
+        interfaces = [i["interface"] for i in device_info]
+        self.assertNotIn("test_iface", interfaces)
 
-        fake_array = array.array("B", response + b"\0" * 4855)
+    @patch("landscape.lib.network.netifaces.ifaddresses")
+    @patch("landscape.lib.network.netifaces.interfaces")
+    def test_skip_iface_with_no_ip(self, mock_interfaces, mock_ifaddresses):
+        mock_interfaces.return_value = _interfaces() + ["test_iface"]
+        mock_ifaddresses.side_effect = lambda iface: (
+             _ifaddresses(iface) if iface in _interfaces() else {AF_UNIX: []})
+        device_info = get_active_device_info()
+        interfaces = [i["interface"] for i in device_info]
+        self.assertNotIn("test_iface", interfaces)
 
-        with patch("array.array") as mock_array:
-            mock_array.return_value = fake_array
-            mock_ioctl.return_value = 0
-            mock_unpack.return_value = (280, 38643456)
-
-            sock = socket.socket(
-                socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_IP)
-            interfaces = list(get_active_interfaces(sock))
-
-        mock_array.assert_called_with("B", ANY)
-        mock_ioctl.assert_called_with(ANY, ANY, ANY)
-        mock_unpack.assert_called_with("iL", ANY)
+    @patch("landscape.lib.network.get_network_interface_speed")
+    @patch("landscape.lib.network.get_flags")
+    @patch("landscape.lib.network.netifaces.ifaddresses")
+    @patch("landscape.lib.network.netifaces.interfaces")
+    def test_skip_iface_down(
+            self, mock_interfaces, mock_ifaddresses, mock_get_flags,
+            mock_get_network_interface_speed):
+        mock_get_network_interface_speed.return_value = (100, True)
+        mock_get_flags.return_value = 0
+        mock_interfaces.return_value = ["test_iface"]
+        mock_ifaddresses.return_value = {
+            AF_LINK: [{"addr": "aa:bb:cc:dd:ee:f0"}],
+            AF_INET: [{"addr": "192.168.1.50", "netmask": "255.255.255.0"}]}
+        device_info = get_active_device_info()
+        interfaces = [i["interface"] for i in device_info]
+        self.assertNotIn("test_iface", interfaces)
 
+    @patch("landscape.lib.network.get_network_interface_speed")
+    @patch("landscape.lib.network.get_flags")
+    @patch("landscape.lib.network.netifaces.ifaddresses")
+    @patch("landscape.lib.network.netifaces.interfaces")
+    def test_no_macaddr_no_netmask_no_broadcast(
+            self, mock_interfaces, mock_ifaddresses, mock_get_flags,
+            mock_get_network_interface_speed):
+        mock_get_network_interface_speed.return_value = (100, True)
+        mock_get_flags.return_value = 4163
+        mock_interfaces.return_value = ["test_iface"]
+        mock_ifaddresses.return_value = {AF_INET: [{"addr": "192.168.0.50"}]}
+        device_info = get_active_device_info(extended=False)
         self.assertEqual(
-            [b"lo", b"eth1:pub", b"br1:metadata",
-             b"br1:0", b"br1", b"br1:priv"],
-            interfaces)
+            device_info,
+            [{
+                'interface': 'test_iface',
+                'ip_address': '192.168.0.50',
+                'broadcast_address': '0.0.0.0',
+                'mac_address': '',
+                'netmask': '',
+                'flags': 4163,
+                'speed': 100,
+                'duplex': True}])
 
     def test_get_network_traffic(self):
         """
@@ -144,11 +240,49 @@ class NetworkInfoTest(BaseTestCase):
         the parsed output against a known sample.
         """
         m = mock_open(read_data=test_proc_net_dev_output)
-        with patch('landscape.lib.network.open', m):
+        # Trusty's version of `mock.mock_open` does not support `readlines()`.
+        m().readlines = test_proc_net_dev_output.splitlines
+        with patch('landscape.lib.network.open', m, create=True):
             traffic = get_network_traffic()
         m.assert_called_with("/proc/net/dev", "r")
         self.assertEqual(traffic, test_proc_net_dev_parsed)
 
+    @patch("landscape.lib.network.get_network_interface_speed")
+    @patch("landscape.lib.network.get_flags")
+    @patch("landscape.lib.network.netifaces.ifaddresses")
+    @patch("landscape.lib.network.netifaces.interfaces")
+    def test_ipv6_skip_link_local(
+            self, mock_interfaces, mock_ifaddresses, mock_get_flags,
+            mock_get_network_interface_speed):
+        mock_get_network_interface_speed.return_value = (100, True)
+        mock_get_flags.return_value = 4163
+        mock_interfaces.return_value = ["test_iface"]
+        mock_ifaddresses.return_value = {
+            AF_LINK: [
+                {"addr": "aa:bb:cc:dd:ee:f1"}],
+            AF_INET6: [
+                {"addr": "fe80::1"},
+                {"addr": "2001::1"}]}
+
+        device_info = get_active_device_info(extended=True)
+
+        self.assertEqual(
+            device_info,
+            [{
+                'interface': 'test_iface',
+                'flags': 4163,
+                'speed': 100,
+                'duplex': True,
+                'ip_addresses': {AF_INET6: [{"addr": "2001::1"}]}}])
+
+    def test_is_up(self):
+        self.assertTrue(is_up(1))
+        self.assertTrue(is_up(1 + 2 + 64 + 4096))
+        self.assertTrue(is_up(0b11111111111111))
+        self.assertFalse(is_up(0))
+        self.assertFalse(is_up(2 + 64 + 4096))
+        self.assertFalse(is_up(0b11111111111110))
+
 
 # exact output of cat /proc/net/dev snapshot with line continuations for pep8
 test_proc_net_dev_output = """\
