@@ -17,15 +17,15 @@ class TemperatureTest(ThermalZoneTest):
         self.assertIs(None, self.successResultOf(self.temperature.run()))
 
     def test_run_adds_header(self):
-        self.write_thermal_zone("THM0", "51 C")
+        self.write_thermal_zone("THM0", "51000")
         self.temperature.run()
         self.assertEqual(self.sysinfo.get_headers(),
-                         [("Temperature", "51 C")])
+                         [("Temperature", "51.0 C")])
 
     def test_ignores_bad_files(self):
         self.write_thermal_zone("THM0", "")
         temperature_path = os.path.join(self.thermal_zone_path,
-                                        "THM0/temperature")
+                                        "THM0/temp")
         file = open(temperature_path, "w")
         file.write("bad-label: 51 C")
         file.close()
@@ -33,14 +33,14 @@ class TemperatureTest(ThermalZoneTest):
         self.assertEqual(self.sysinfo.get_headers(), [])
 
     def test_ignores_unknown_formats(self):
-        self.write_thermal_zone("THM0", "FOO C")
+        self.write_thermal_zone("THM0", "FOO")
         self.temperature.run()
         self.assertEqual(self.sysinfo.get_headers(), [])
 
     def test_picks_highest_temperature(self):
-        self.write_thermal_zone("THM0", "51 C")
-        self.write_thermal_zone("THM1", "53 C")
-        self.write_thermal_zone("THM2", "52 C")
+        self.write_thermal_zone("THM0", "51000")
+        self.write_thermal_zone("THM1", "53000")
+        self.write_thermal_zone("THM2", "52000")
         self.temperature.run()
         self.assertEqual(self.sysinfo.get_headers(),
-                         [("Temperature", "53 C")])
+                         [("Temperature", "53.0 C")])
